#include "testApp.h"

#define AUDIO_BUFFER_SIZE 1024
#define FFT_SIZE AUDIO_BUFFER_SIZE
#define NUM_DIALS 8

//--------------------------------------------------------------
void testApp::setup()
{
	// ofxStkPrintAll();
	// ofxStkPrintInstrmnt("dummy");
	// ofxStkPrintInstrmnt(123);

	// init parameter
	const vector<string> &names = ofxStkGetInstrmntNames();
	mNumStkInstrmnt = names.size();
	mCurrentInstrmnt = 1;
	
	// STK instrmnt
	insts.resize(mNumStkInstrmnt);
	for(int i = 0; i < mNumStkInstrmnt; i++) {
		insts[i] = ofxStkNewInstrmnt(names[i]);
	}

	// GUI setting
	sigAreaX = (int)(0.3 * ofGetWidth());
	sigAreaY = (int)(0.05 * ofGetHeight());
	sigAreaW = (int)(0.65 * ofGetWidth());
	sigAreaH = (int)(0.30 * ofGetHeight());

	oscilloAreaX = (int)(0.3 * ofGetWidth());
	oscilloAreaY = (int)(0.40 * ofGetHeight());
	oscilloAreaW = (int)(0.65 * ofGetWidth());
	oscilloAreaH = (int)(0.30 * ofGetHeight());
	
	buttonX = (int)(0.05 * ofGetWidth());
	buttonY = (int)(0.05 * ofGetHeight());
	buttonSize = (int)(0.018 * ofGetHeight());

	dialX = (int)(0.3 * ofGetWidth());
	dialY = (int)(0.75 * ofGetHeight());
	dialSize = (int)(0.03 * ofGetHeight());
	dialLocationMargin = (int)(0.03 * ofGetHeight());

	// setup of background
    ofBackground(50, 50, 50);

	// setup buttons
	buttons.resize(mNumStkInstrmnt);
	for(int i = 0; i < mNumStkInstrmnt; i++) {
		buttons[i].setup(buttonX, buttonY + (buttonSize << 1) * i,
						 buttonSize, buttonSize, true);
		buttons[i].addListener(this);
	}

	// setup dials
	dials.resize(NUM_DIALS);
	for(int i = 0; i < NUM_DIALS; i++) {
		int xx = dialX + dialLocationMargin + ((dialSize*4.0 + dialLocationMargin) * (i % 4));
		int yy = dialY + dialLocationMargin + ((dialSize*2.3 + dialLocationMargin) * (i / 4));
		dials[i].setup(xx, yy, dialSize, true);
		dials[i].addListener(this);
	}
	
    // audio setup
    ofSoundStreamSetup(2, 0, this, 44100, AUDIO_BUFFER_SIZE, 4);
	lrAudio.assign(AUDIO_BUFFER_SIZE, 0.0);

	// setup Maxim fft
	mFftPowdB = new float[FFT_SIZE];
	mfft.setup(FFT_SIZE, FFT_SIZE>>1, FFT_SIZE>>2);
}

//--------------------------------------------------------------
void testApp::update(){

}

//--------------------------------------------------------------
void testApp::draw(){
	ofPushStyle();
	ofNoFill();

	//ofSetFrameRate(30);
	
	ofDrawBitmapString(ofToString(ofGetFrameRate())+"fps", 10, 15);

	const vector<string> &names = ofxStkGetInstrmntNames();
	
	// draw button names
	for(int i = 0; i < mNumStkInstrmnt; i++) {
		ofDrawBitmapString(names[i],
						   buttonX + (buttonSize << 1),
						   buttonY + (buttonSize * 0.8) + (buttonSize << 1) *i);
	}
	
	// draw dial names
	const vector<ControlChangeInfo> &infos = ofxStkControlChangeInfos(names[mCurrentInstrmnt]);
	for(int i = 0; i < infos.size(); i++)
	{
		int bitmap_font_size = 5;
		string short_name = infos[i].name.substr(0, 8);
		int name_length_offset = (dialSize >> 1) - ((short_name.size() * bitmap_font_size) >> 1);
		//int name_length_offset = (dialSize >> 1) - ((infos[i].name.size() * bitmap_font_size) >> 1);
		int xx = dialX + dialLocationMargin + ((dialSize*4.0 + dialLocationMargin) * (i % 4));
		int yy = dialY + dialLocationMargin + ((dialSize*2.3 + dialLocationMargin) * (i / 4));
		//ofDrawBitmapString(infos[i].name, xx + name_length_offset, yy);
		ofDrawBitmapString(short_name, xx + name_length_offset, yy);
	}
	
	// draw signal scope rect
	{
		ofColor(100, 100, 100); 
		ofRect(sigAreaX, sigAreaY, sigAreaW, sigAreaH);
	}

	// draw signal scope
	{
		int hh = sigAreaH >> 1;
		ofPushMatrix();
		ofTranslate(sigAreaX, sigAreaY);
		ofColor(150, 150, 10);
		ofBeginShape();
		for(int i = 0; i < AUDIO_BUFFER_SIZE; i++)
		{
			double x = ofMap(i, 0, AUDIO_BUFFER_SIZE, 0, sigAreaW);
			double y = hh + (sigAreaH * lrAudio[i]);
			ofVertex(x, y);
		}
		ofEndShape();
		ofPopMatrix();
	}
	
	// draw oscilloscope rect
	{
		ofColor(100, 100, 100); 
		ofRect(oscilloAreaX, oscilloAreaY, oscilloAreaW, oscilloAreaH);
	}

	// draw oscilloscope
	{
		ofPushMatrix();
		ofTranslate(oscilloAreaX, oscilloAreaY);
		ofColor(150, 150, 10);
		ofBeginShape();
		for(int i = 0; i < FFT_SIZE>>1; i++)
		{
			double x = ofMap(i, 0, FFT_SIZE>>1, 0, oscilloAreaW);
			double drawPower = (mFftPowdB[i] * oscilloAreaH * 0.02);
			if(drawPower > oscilloAreaH) drawPower = oscilloAreaH;
			ofVertex(x, oscilloAreaH - drawPower);
		}
		ofEndShape();
		ofPopMatrix();
	}
	
	ofPopStyle();
}

void testApp::audioOut(float *output, int size, int channels)
{
    for(int i = 0; i < size; i++) {
        double v = insts[mCurrentInstrmnt]->tick();
        output[i * channels] = v;
        output[i * channels + 1] = v;
		lrAudio[i] = v;

		// ---- maxi fft
		if(mfft.process(v)) {
			float *magdb = mfft.magsToDB();
			for(int i = 0; i < (FFT_SIZE>>1); i++) {
				mFftPowdB[i] = magdb[i];
			}
		}
    }
}

void testApp::buttonNotify(ofxSimpleButtonEvent event)
{
	for(int i = 0; i < mNumStkInstrmnt; i++) {
		if(event.button == &buttons[i]) {
			// const vector<string> &names = ofxStkGetInstrmntNames();
			// const string nm = names[i];
			// const vector<ControlChangeInfo> &infos = ofxStkControlChangeInfos(nm);
			// printf("Instrmnt : %s\n", nm.c_str());
			// for(vector<ControlChangeInfo>::const_iterator itr = infos.begin(); itr != infos.end(); itr++) {
			// 	ControlChangeInfo info = ((ControlChangeInfo)*itr);
			// 	printf("%d, %s\n", info.number, info.name.c_str());
			// }
			
			if(event.type == BUTTON_PRESSED) {
				insts[i]->noteOn(220, 0.5);
			} else if(event.type == BUTTON_RELEASED) {
				insts[i]->noteOff(0.5);
			}

			mCurrentInstrmnt = i;

			break;
		}
	}
}

void testApp::dialNotify(ofxSimpleDialEvent &event)
{
	for(int i = 0; i < NUM_DIALS; i++) {
		if(event.dial == &dials[i]) {
			const vector<ControlChangeInfo> &cc_infos = ofxStkControlChangeInfos(mCurrentInstrmnt);
			if(i < cc_infos.size()) {
				insts[mCurrentInstrmnt]->controlChange(cc_infos[i].number, event.value * 128);
				printf("dial number=%d, cc_name=%s, cc_number=%d, value=%f\n",
					   i, cc_infos[i].name.c_str(), cc_infos[i].number, event.value);
				break;
			}
		}
	}
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo){ 

}
